unit StringHash;
{
    TStringHash v0.9
    A Perl-style hash class accessed like an array, but with a string as
    the index. Note that an _exact_ match for the index is required, so
    'this', 'This', ' this' and 'this ' are all different. The key '' is
    invalid and will be rejected. 
    
    Copyright (C) 2000 Ciaran McCreesh

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{ TODO 2 -okeesh -cbug fix : Add code for preventing '' as name }
{ TODO 4 -okeesh -cperformance : Change to bsearch }
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TStringHash = class(TComponent)
  private
    fKeys: array of string;
    fValues: array of string;
    fCount: integer;
    function fFindItem(const key: string): integer;
    function fGetItems(const key: string): string;
    procedure fSetItems(const key, Value: string);
    function fGetKeyByIndex(const i: integer): string;
    function fGetValueByIndex(const i: integer): string;
    procedure fSetKeyByIndex(const i: integer; const Value: string);
    procedure fSetValueByIndex(const i: integer; const Value: string);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(const key: string);
    function Exists(const key: string): boolean;
    property Items[const key: string]: string read fGetItems write fSetItems; default;
    property KeyByIndex[const i: integer]: string read fGetKeyByIndex write fSetKeyByIndex;
    property ValueByIndex[const i: integer]: string read fGetValueByIndex write fSetValueByIndex;
    property ItemCount: integer read fCount;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterNoIcon([TStringHash]);
end;

{ TStringHash }

procedure TStringHash.Clear;
begin
  if (self.fCount > 0) then begin
    SetLength(self.fKeys, 0);
    SetLength(self.fValues, 0);
    self.fCount := 0;
  end;
end;

constructor TStringHash.Create(AOwner: TComponent);
begin
  inherited;
  SetLength(self.fKeys, 0);
  SetLength(self.fValues, 0);
  self.fCount := 0;
end;

procedure TStringHash.Delete(const key: string);
var
  i: integer;
begin
  i := self.fFindItem(key);
  if (i >= 0) then begin
    self.fKeys[i] := self.fKeys[self.fCount - 1];
    self.fValues[i] := self.fValues[self.fCount - 1];
    dec(self.fCount);
    SetLength(self.fKeys, self.fCount);
    SetLength(self.fValues, self.fCount);
  end;
end;

destructor TStringHash.Destroy;
begin
  inherited;
end;

function TStringHash.Exists(const key: string): boolean;
begin
  result := self.fFindItem(key) >= 0;
end;

function TStringHash.fFindItem(const key: string): integer;
var
  found: boolean;
begin
  found := false;
  result := 0;
  while ((not found) and (result < self.fCount)) do
    if (self.fKeys[result] = key) then
      found := true
    else
      inc(result);
  if (not found) then
    result := -1;
end;

function TStringHash.fGetItems(const key: string): string;
var
  i: integer;
begin
  i := self.fFindItem(key);
  if (i = -1) then
    result := ''
  else
    result := self.ValueByIndex[i];
end;

procedure TStringHash.fSetItems(const key, Value: string);
var
  i: integer;
begin
  i := self.fFindItem(key);
  if (i = -1) then begin
    inc(self.fCount);
    SetLength(self.fKeys, self.fCount);
    SetLength(self.fValues, self.fCount);
    self.ValueByIndex[self.fCount - 1] := Value;
    self.KeyByIndex[self.fCount - 1] := key;
  end else
    self.ValueByIndex[i] := Value;
end;

function TStringHash.fGetKeyByIndex(const i: integer): string;
begin
  result := self.fKeys[i];
end;

function TStringHash.fGetValueByIndex(const i: integer): string;
begin
  result := self.fValues[i];
end;

procedure TStringHash.fSetKeyByIndex(const i: integer; const Value: string);
begin
  self.fKeys[i] := Value;
end;

procedure TStringHash.fSetValueByIndex(const i: integer; const Value: string);
begin
  self.fValues[i] := Value;
end;

end.
