unit XMLTreeView;

{
    Tree View for HTML-Kit (Display Component)
    This component is used for the main display. It is easiest to install
    it through the package provided, although you can do a manual install
    as you would for any other component. This component will register on
    the 'Parsers' page by default -- it isn't a parser but everything else
    used by this project goes there... Feel free to change the page to fit
    in with your existing layout.

    Copyright (C) 2000 Ciaran McCreesh

    Get the latest version and futher information from the project website,
    <http://hktreeview.sourceforge.net/>, or email the author
    <keesh@users.sourceforge.net>. This project is known to compile under
    Delphi 5 Professional, older versions of Delphi may reject it.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TTreeItem = record
    Caption: string;
    Indent: byte;
    ImageIndex: integer;
    FontStyle: TFontStyles;
  end;

  TXMLTreeView = class(TCustomPanel)
  private
    fItems: array of TTreeItem;
    fCanRepaint: boolean;
    fItemHeight: integer;
    fIndentWidth: integer;
    fImages: TImageList;
    procedure SetItemHeight(const Value: integer);
    procedure SetIndentWidth(const Value: integer);
    procedure SetImages(const Value: TImageList);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure AddItem(const Caption: string; const Indent, ImageIndex: integer;
      const FontStyle: TFontStyles);
  published
    property CanRepaint: boolean read fCanRepaint write fCanRepaint;
    property ItemHeight: integer read FItemHeight write SetItemHeight;
    property IndentWidth: integer read FIndentWidth write SetIndentWidth;
    property Color;
    property Images: TImageList read FImages write SetImages;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Parsers', [TXMLTreeView]);
end;

{ TXMLTreeView }

procedure TXMLTreeView.AddItem(const Caption: string; const Indent,
  ImageIndex: integer; const FontStyle: TFontStyles);
var
  r: TTreeItem;
begin
  SetLength(self.fItems, High(self.fItems) + 2);
  r.Caption := Caption;
  r.Indent := Indent;
  r.ImageIndex := ImageIndex;
  r.FontStyle := FontStyle;
  self.fItems[High(self.fItems)] := r;
  self.Height := self.ItemHeight * (High(self.fItems) + 1);
  self.Invalidate;
end;

constructor TXMLTreeView.Create(AOwner: TComponent);
begin
  inherited;
  self.Align := alTop;
  self.Caption := '';
  self.CanRepaint := true;
  self.fItemHeight := 20;
  self.fIndentWidth := 16;
  self.BevelInner := bvNone;
  self.BevelOuter := bvNone;
  if (csDesigning in self.ComponentState) then
    self.Height := 20
  else
    self.Height := 0;
end;

destructor TXMLTreeView.Destroy;
begin
  inherited;
end;

procedure TXMLTreeView.Paint;
var
  lc: integer;
  ih: array[-1..256] of integer; 
begin
  inherited;
  if (self.CanRepaint) then begin
    ZeroMemory(@ih[0], sizeof(ih));
    for lc := Low(self.fItems) to High(self.fItems) do begin
      { Line } { TODO : make this more efficient }
      if (ih[self.fItems[lc].indent - 1] > 0) then begin
        self.Canvas.Pen.Color := clBtnFace;
        self.Canvas.Pen.Width := 3;
        self.Canvas.MoveTo(self.IndentWidth * self.fItems[lc].indent - 2,
          lc * self.ItemHeight + (self.ItemHeight div 2));
        self.Canvas.LineTo(self.Canvas.PenPos.x - (self.IndentWidth div 2) + 2,
          self.Canvas.PenPos.y);
        self.Canvas.LineTo(self.Canvas.PenPos.x,
          ih[self.fItems[lc].indent - 1]);
      end;
      ih[self.fItems[lc].indent] := (lc + 1) * self.ItemHeight;
      { Text }
      self.Canvas.Brush.Color := self.Color;
      self.Canvas.Font.Style := self.fItems[lc].FontStyle;
      self.Canvas.TextOut(
        ((self.fItems[lc].Indent + 1) * self.IndentWidth),
        self.ItemHeight * lc + 1,
        self.fItems[lc].Caption);
      { Icon }
      if (Assigned(self.Images)) then
        self.Images.Draw(
          self.Canvas,
          self.fItems[lc].Indent * self.IndentWidth,
          self.ItemHeight * lc + 1,
          self.fItems[lc].ImageIndex);
    end;
  end;
end;

procedure TXMLTreeView.SetImages(const Value: TImageList);
begin
  fImages := Value;
  self.Invalidate;
end;

procedure TXMLTreeView.SetIndentWidth(const Value: integer);
begin
  self.fIndentWidth := Value;
  self.Invalidate;
end;

procedure TXMLTreeView.SetItemHeight(const Value: integer);
begin
  self.fItemHeight := Value;
  self.Invalidate;
end;

end.


