unit cmtreeview1;

{
    Tree View for HTML-Kit (Main Form)
    This HTML-Kit plugin displays documents in a 'tree' view.

    Copyright (C) 2000 Ciaran McCreesh

    Get the latest version and futher information from the project website,
    <http://hktreeview.sourceforge.net/>, or email the author
    <keesh@users.sourceforge.net>. This project is known to compile under
    Delphi 5 Professional, older versions of Delphi may reject it.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ActnList, ToolWin, ComCtrls, ImgList, ComObj, HKPCore,
  ParserStream, ParserMemoryStream, SParser, SXMLParser, StdCtrls, ExtCtrls,
  Grids, XMLTreeView, StringHash;

type
  ETreeError = class(Exception);
  ETreeErrorNest = class(ETreeError);

  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuFileExit: TMenuItem;
    MenuHelp: TMenuItem;
    MenuHelpDocumentation: TMenuItem;
    MenuHelpWebsite: TMenuItem;
    MenuHelpLatestVersion: TMenuItem;
    MenuHelpN1: TMenuItem;
    MenuHelpAbout: TMenuItem;
    Actions: TActionList;
    AExit: TAction;
    ADocumentation: TAction;
    AWebsite: TAction;
    ALatestVersion: TAction;
    AAbout: TAction;
    StatusBar: TStatusBar;
    ToolImages: TImageList;
    AOpenURL: TAction;
    XMLParser: TSXMLParser;
    ParserStream: TParserMemoryStream;
    ToolBar: TToolBar;
    ToolBarExit: TToolButton;
    ToolBarDocumentation: TToolButton;
    ToolBarWebsite: TToolButton;
    ToolBarAbout: TToolButton;
    TreeImages: TImageList;
    ToolBarS2: TToolButton;
    ScrollBox: TScrollBox;
    Tree: TXMLTreeView;
    procedure AExitExecute(Sender: TObject);
    procedure AAboutExecute(Sender: TObject);
    procedure AWebsiteExecute(Sender: TObject);
    procedure ADocumentationExecute(Sender: TObject);
    procedure ALatestVersionExecute(Sender: TObject);
    procedure AOpenURLExecute(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ParseDisplay;
    procedure HandleHints(Sender: TObject);
  private
    fURL: string;
  public
    property URL: string read fURL write fURL;
  end;

var
  MainForm: TMainForm;

implementation

uses cmtreeview2, cmtreeview3;

{$R *.DFM}

procedure TMainForm.AExitExecute(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TMainForm.AAboutExecute(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

procedure TMainForm.AWebsiteExecute(Sender: TObject);
begin
  MainForm.URL := 'http://hktreeview.sourceforge.net/';
  AOpenURL.Execute;
end;

procedure TMainForm.ADocumentationExecute(Sender: TObject);
begin
  MainForm.URL := ExtractFilePath(Application.ExeName) +
    'docs\cmtreeview\manual.html';
  AOpenURL.Execute;
end;

procedure TMainForm.ALatestVersionExecute(Sender: TObject);
begin
  MainForm.URL := 'http://sourceforge.net/projects/hktreeview/';
  AOpenURL.Execute;
end;

procedure TMainForm.AOpenURLExecute(Sender: TObject);
var
  v: Variant;
begin
  v := CreateOleObject('htmlkit.HTMLKitPluginsCore');
  v.hkp_FuncOpen(hkp_f_ExecuteAndWait);
  v.hkp_DataAdd('FUNC_PARAM1', MainForm.URL);
  v.hkp_DataAdd('FUNC_PARAM2', '');
  v.hkp_DataAdd('FUNC_PARAM3', '');
  v.hkp_DataAdd('FUNC_PARAM4', '0');
  v.hkp_FuncRun;
  v.hkp_FuncClose;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  Application.OnHint := HandleHints;
  ParseDisplay;
end;

procedure TMainForm.ParseDisplay;
{ This does the displaying... It's the difficult bit. Needs rewriting
  sometime. }
var
  v: Variant;
  s, t: string;
  ds: integer;
  indent: integer;
  grpid: string;
  icons: TStringHash;
  bmp: TBitmap;
  ii: integer;
  ek: TSXMLElementKind;

  function fStripQuotes(const s: string): string;
  begin
    if ((length(s) > 1) and (s[1] in ['''', '"']) and (s[length(s)] = s[1])) then
      result := copy(s, 2, length(s) - 2)
    else
      result := s;
  end;

  function fMakeNice(const s: string): string;
  { This is like s/\s*/ /g; in Perl -- replace whitespace* with
    ' '. There's probably a more efficient way of doing this. }
  var
    lc, ps: integer;
    ws: boolean;
  begin
    result := '';
    if (s <> '') then begin
      ws := s[1] in cWhitespace;
      ps := 1;
      for lc := 1 to length(s) do begin
        if ((s[lc] in cWhitespace) and (not ws)) then begin
          ws := true;
          result := result + copy(s, ps, lc - ps) + ' ';
        end else if ((not (s[lc] in cWhitespace)) and (ws)) then begin
          ws := false;
          ps := lc;
        end;
      end;
      if (not ws) then
        result := result + copy(s, ps, length(s)); { whatever's left }
    end else
      result := '';
  end;

  procedure fAddParameters;
  { Adds parameters. }
  var
    lc: integer;
  begin
    for lc := 0 to (XMLParser.Attributes.ItemCount - 1) do begin
      ii := StrToIntDef(icons['attributes.' + XMLParser.Attributes.KeyByIndex[lc]], 9);
      Tree.AddItem(fMakeNice(XMLParser.Attributes.KeyByIndex[lc]),
        indent + 1, ii, []);
      Tree.AddItem(fMakeNice(XMLParser.Attributes.ValueByIndex[lc]),
        indent + 2, 10, [fsItalic]);
    end;
  end;

  function fIsEmpty(const s: string): boolean;
  const
    cEmpties = '#img#br#hr#link#meta#base#basefont#area#frame#iframe#param#';
  begin
    result := pos('#' + s + '#', cEmpties) > 0;
  end;

begin
  ScrollBox.Hide; { Reduce flickers }
  Tree.CanRepaint := false; { Same again }

  { First, get the data from config\cmtreeview\elements.xml }
  ParserStream.LoadFromFile(ExtractFilePath(Application.ExeName) +
    'config\cmtreeview\elements.xml');
  ParserStream.Seek(0, soFromBeginning);
  icons := TStringHash.Create(MainForm);
  icons.Clear;
  try
    try
      grpid := '';
      while (XMLParser.NextElement) do begin
        if (XMLParser.ElementName = 'group') then
          grpid := XMLParser.Attributes['title']
        else if (XMLParser.ElementName = 'item') then
          if (XMLParser.ElementKind = xeStart) then begin
            s := lowercase(XMLParser.Attributes['name']);
            t := lowercase(XMLParser.Attributes['kind']);
          end else if (XMLParser.ElementKind = xeEnd) then
            if (fStripQuotes(t) = 'index') then
              icons[fStripQuotes(grpid) + '.' + fStripQuotes(s)] := XMLParser.ElementText
            else begin
              bmp := TBitmap.Create;
              bmp.LoadFromFile(ExtractFilePath(Application.ExeName) +
                'lib\cmtreeview\' + XMLParser.ElementText);
              bmp.Transparent := true;
              bmp.TransparentMode := tmAuto;
              try
                TreeImages.Add(bmp, bmp);
                icons[fStripQuotes(grpid) + '.' + fStripQuotes(s)] :=
                  inttostr(TreeImages.Count - 1);
              finally
                bmp.Free;
              end;
            end;
      end;
    except
      on e:Exception do begin
        ErrorForm.LabelTopText.Caption :=
          'There was an error using the config file. Please restore the file ' +
          'to its original state and try again.';
        ErrorForm.LabelErrorKind.Caption := format('Error %s: %s',
          [e.ClassName, e.Message]);
        ErrorForm.LabelErrorLocation.Caption := format('elements.xml, Near Line %d',
          [XMLParser.LineNumber]);
        ErrorForm.ShowModal;
        Icons.Clear;
      end;
    end;

    { Now get the data and display it }
    v := CreateOleObject('htmlkit.HTMLKitPluginsCore');
    v.hkp_DataGet(hkp_INPUT, s);
    ParserStream.LoadFromString(s);
    ds := length(s); { Save length for % display }
    s := ''; { Free memory as soon as possible }
    ParserStream.Seek(0, soFromBeginning);
    indent := 0;
    try
      while (XMLParser.NextElement) do begin
        StatusBar.Panels[0].Text := IntToStr(round(XMLParser.Position / ds * 100)) +
          '% Complete'; { Slow machine, 5MByte XML database... That's why :) }
        StatusBar.Update; { Force refresh if changed }
        { Add text }
        s := fMakeNice(XMLParser.ElementText);
        if (s <> '') then begin
          Tree.AddItem('''' + s + '''', indent + 1, 11, [fsBold]);
          s := '';
        end;
        ek := XMLParser.ElementKind;
        if ((ek = xeStart) and (fIsEmpty(XMLParser.ElementName))) then
          ek := xeEmpty;

        case (ek) of
          xeStart: begin { Add item and indent }
            inc(indent);
            ii := StrToIntDef(icons['elements.' + lowercase(XMLParser.ElementName)], -2);
            if (ii = -2) then
              ii := ord(XMLParser.ElementKind);
            Tree.AddItem(XMLParser.ElementName, indent, ii, []);
            fAddParameters;
          end;
          xeEnd: { Outdent }
            dec(indent);
          else begin { Add item }
            inc(indent);
            ii := StrToIntDef(icons['elements.' + lowercase(XMLParser.ElementName)], -2);
            if (ii = -2) then
              ii := ord(XMLParser.ElementKind);
            Tree.AddItem(XMLParser.ElementName, indent, ii, []);
            fAddParameters;
            dec(indent);
          end;
        end;
      end;
    except
      on e:Exception do begin
        ErrorForm.LabelErrorKind.Caption := format('Error %s: %s',
          [e.ClassName, e.Message]);
        ErrorForm.LabelErrorLocation.Caption := format('Current Document, Near Line %d',
          [XMLParser.LineNumber]);
        ErrorForm.ShowModal;
        Application.Terminate;
      end;
    end;
  finally
    icons.Free;
  end;
  StatusBar.Panels[0].Text := 'Document Parsed Successfully';
  Tree.CanRepaint := true;
  ScrollBox.Show;
  Tree.Invalidate;
end;

procedure TMainForm.HandleHints(Sender: TObject);
begin
  StatusBar.Panels[0].Text := GetLongHint(Application.Hint);
  Application.Hint := GetShortHint(Application.Hint);
  MainForm.ShowHint := Application.Hint <> '';
end;

end.


